@@ -52,23 +52,28 @@
 #ifndef INFINITY
 #define INFINITY (HUGE_VAL)
 #endif
+#include <stdint.h>
 #include "common.h"
+#include "pow_int.h"
 
 static inline double
-gd_common (int dim, const signed char *minmax,
-           const double *points_a, int size_a,
-           const double *points_r, int size_r,
-           bool plus, bool psize, unsigned int p)
+gd_common (int dim, const signed char * restrict minmax,
+           const double * restrict points_a, int size_a,
+           const double * restrict points_r, int size_r,
+           bool plus, bool psize, uint_fast8_t p)
 {
     if (size_a == 0) return INFINITY;
+    ASSUME(size_a > 0);
+    ASSUME(size_r > 0);
+    ASSUME(dim >= 2);
+    ASSUME(dim <= 32);
 
-    int a, r, d;
     double gd = 0;
-    for (a = 0; a < size_a; a++) {
+    for (int a = 0; a < size_a; a++) {
         double min_dist = INFINITY;
-        for (r = 0; r < size_r; r++) {
+        for (int r = 0; r < size_r; r++) {
             double dist = 0.0;
-            for (d = 0; d < dim; d++) {
+            for (int d = 0; d < dim; d++) {
                 if (minmax[d] == 0) continue;
                 double a_d = points_a[a * dim + d];
                 double r_d = points_r[r * dim + d];
@@ -84,9 +89,13 @@ gd_common (int dim, const signed char *minmax,
             if (dist < min_dist) min_dist = dist;
         }
         // Here we calculate the actual Euclidean distance.
-        min_dist = (double) sqrtl(min_dist);
-
-        gd += (p == 1) ? min_dist : (double) powl(min_dist, p);
+        if (p == 1)
+            min_dist = (double) sqrtl(min_dist);
+        else if (p % 2 == 0)
+            min_dist = pow_uint(min_dist, p/2);
+        else
+            min_dist = pow_uint((double) sqrtl(min_dist), p);
+        gd += min_dist;
     }
 
     if (p == 1)
@@ -98,79 +107,76 @@ gd_common (int dim, const signed char *minmax,
 }
 
 static inline double
-GD_minmax (int dim, const signed char *minmax,
-           const double *points_a, int size_a,
-           const double *points_r, int size_r)
+GD_minmax (int dim, const signed char * restrict minmax,
+           const double * restrict points_a, int size_a,
+           const double * restrict points_r, int size_r)
 {
     return gd_common (dim, minmax,
                       points_a, size_a,
                       points_r, size_r,
                       /*plus=*/false, /*psize=*/false,
-                      /*p=*/1);
+                      /*p=*/(uint_fast8_t)1);
 }
 
 static inline double
-IGD_minmax (int dim, const signed char *minmax,
-            const double *points_a, int size_a,
-            const double *points_r, int size_r)
+IGD_minmax (int dim, const signed char * restrict minmax,
+            const double * restrict points_a, int size_a,
+            const double * restrict points_r, int size_r)
 {
     return gd_common (dim, minmax,
                       points_r, size_r,
                       points_a, size_a,
                       /*plus=*/false, /*psize=*/false,
-                      /*p=*/1);
+                      /*p=*/(uint_fast8_t)1);
 }
 
-_no_warn_unused static double
-IGD (const double *data, int nobj, int npoints, const double *ref, int ref_size,
-     const bool * maximise)
+_attr_maybe_unused static double
+IGD (const double * restrict data, int nobj, int npoints,
+     const double * restrict ref, int ref_size,
+     const bool *  restrict maximise)
 {
     const signed char *minmax = minmax_from_bool(nobj, maximise);
     double value = IGD_minmax (nobj, minmax, data, npoints, ref, ref_size);
     free ((void *)minmax);
-    return(value);
+    return value;
 }
 
 static inline double
-GD_p (int dim, const signed char *minmax,
-      const double *points_a, int size_a,
-      const double *points_r, int size_r, unsigned int p)
+GD_p (int dim, const signed char * restrict minmax,
+      const double * restrict points_a, int size_a,
+      const double * restrict points_r, int size_r, unsigned int p)
 {
     return gd_common (dim, minmax,
                       points_a, size_a,
                       points_r, size_r,
-                      /*plus=*/false, /*psize=*/true,
-                      p);
+                      /*plus=*/false, /*psize=*/true, (uint_fast8_t)p);
 }
 
 static inline double
-IGD_p (int dim, const signed char *minmax,
-       const double *points_a, int size_a,
-       const double *points_r, int size_r, unsigned int p)
+IGD_p (int dim, const signed char * restrict minmax,
+       const double * restrict points_a, int size_a,
+       const double * restrict points_r, int size_r, unsigned int p)
 {
     return gd_common (dim, minmax,
                       points_r, size_r,
                       points_a, size_a,
-                      /*plus=*/false, /*psize=*/true,
-                      p);
+                      /*plus=*/false, /*psize=*/true, (uint_fast8_t) p);
 }
 
 static inline double
-IGD_plus_minmax (int dim, const signed char *minmax,
-                 const double *points_a, int size_a,
-                 const double *points_r, int size_r)
+IGD_plus_minmax (int dim, const signed char * restrict minmax,
+                 const double * restrict points_a, int size_a,
+                 const double * restrict points_r, int size_r)
 {
     return gd_common (dim, minmax,
                       points_r, size_r,
                       points_a, size_a,
-                      /*plus=*/true, /*psize=*/true,
-                      /*p=*/1);
-
+                      /*plus=*/true, /*psize=*/true, /*p=*/(uint_fast8_t)1);
 }
 
-_no_warn_unused static double
-IGD_plus (const double *data, int nobj, int npoints, const double *ref, int ref_size,
-          const bool * maximise)
+_attr_maybe_unused static double
+IGD_plus (const double * restrict data, int nobj, int npoints, const double * restrict ref, int ref_size,
+          const bool * restrict  maximise)
 {
     const signed char *minmax = minmax_from_bool(nobj, maximise);
     double value = IGD_plus_minmax (nobj, minmax, data, npoints, ref, ref_size);
@@ -179,28 +185,28 @@ IGD_plus (const double *data, int nobj, int npoints, const double *ref, int ref_
 }
 
 static inline double
-avg_Hausdorff_dist_minmax (int dim, const signed char *minmax,
-                           const double *points_a, int size_a,
-                           const double *points_r, int size_r, unsigned int p)
+avg_Hausdorff_dist_minmax (int dim, const signed char * restrict minmax,
+                           const double * restrict points_a, int size_a,
+                           const double * restrict points_r, int size_r,
+                           unsigned int p)
 {
     double gd_p = gd_common (dim, minmax,
                              points_a, size_a,
                              points_r, size_r,
-                             /*plus=*/false, /*psize=*/true,
-                             p);
+                             /*plus=*/false, /*psize=*/true, (uint_fast8_t)p);
 
     double igd_p = gd_common (dim, minmax,
                               points_r, size_r,
                               points_a, size_a,
-                              /*plus=*/false, /*psize=*/true,
-                              p);
-    return MAX (gd_p, igd_p);
+                              /*plus=*/false, /*psize=*/true, (uint_fast8_t)p);
+    return MAX(gd_p, igd_p);
 }
 /* TODO: Implement p=INFINITY See [4] */
 
-_no_warn_unused static double
-avg_Hausdorff_dist (const double *data, int nobj, int npoints, const double *ref, int ref_size,
-                    const bool * maximise, unsigned int p)
+_attr_maybe_unused static double
+avg_Hausdorff_dist (const double * restrict data, int nobj, int npoints,
+                    const double * restrict ref, int ref_size,
+                    const bool * restrict maximise, unsigned int p)
 {
     const signed char * minmax = minmax_from_bool(nobj, maximise);
     double value = avg_Hausdorff_dist_minmax (nobj, minmax, data, npoints, ref, ref_size, p);
