@@ -48,14 +48,16 @@
 #include <string.h>
 #include <ctype.h> // for isspace()
 #include <float.h>
-
 #include <unistd.h>  // for getopt()
 #include <getopt.h> // for getopt_long()
 #include <math.h> // for log10()
+
 #include "nondominated.h"
+
 #define READ_INPUT_WRONG_INITIAL_DIM_ERRSTR "either -o, --obj, -u, --upper or -l, --lower"
 #include "cmdline.h"
 
+
 static int verbose_flag = false;
 static bool union_flag = false;
 static bool check_flag = true;
@@ -81,8 +83,9 @@ OPTION_HELP_STR
 OPTION_VERSION_STR
 " -v, --verbose       print some extra information;                         \n"
 OPTION_QUIET_STR
-"     --no-check      do not check nondominance of sets (faster but unsafe);\n"
+OPTION_NOCHECK_STR
 OPTION_OBJ_STR
+OPTION_MAXIMISE_STR
 " -u, --upper-bound POINT defines an upper bound to check, e.g. \"10 5 30\";\n"
 " -l, --lower-bound POINT defines a lower bound to check;\n"
 " -U, --union         consider each file as a whole approximation set,      \n"
@@ -128,87 +131,19 @@ read_range (char *str, double *lower, double *upper)
     return true;
 }
 
-/* FIXME: Handle "1 NAN 3", so NAN means: use the minimum/maximum for this
-   dimension. */
-static double *
-read_point (char * str, int *nobj)
-{
-    int k = 0, size = 10;
-
-    double * point = malloc (size * sizeof(double));
-    char * endp = str;
-    char * cursor;
-    do {
-        cursor = endp;
-        if (k == size) {
-            size += 10;
-            point = realloc (point, size * sizeof(double));
-        }
-        point[k] = strtod (cursor, &endp);
-        k++;
-    } while (cursor != endp);
-
-    // not end of string: error
-    while (*cursor != '\0') {
-        if (!isspace(*cursor)) return NULL;
-        cursor++;
-    }
-
-    // no number: error
-    if (k == 1) return NULL;
-
-    *nobj = k - 1;
-    return point;
-}
-
 static double *
 robust_read_point (char * str, int *nobj, const char * errmsg)
 {
     double * point = read_point(str, nobj);
-    if (point == NULL) {
-        errprintf (errmsg, optarg);
-        exit (EXIT_FAILURE);
-    }
+    if (point == NULL)
+        fatal_error(errmsg, optarg);
     return point;
 }
 
-static void
-data_bounds (double **minimum, double **maximum,
-             const double *data, int nobj, int rows)
-{
-    int k, r;
-
-#ifndef INFINITY
-#define INFINITY DBL_MAX
-#endif
-#define NEG_INFINITY -DBL_MAX
-
-    if (*minimum == NULL) {
-        *minimum = malloc (nobj * sizeof(double));
-        for (k = 0; k < nobj; k++)
-            (*minimum)[k] = INFINITY;
-    }
-
-    if (*maximum == NULL) {
-        *maximum = malloc (nobj * sizeof(double));
-        for (k = 0; k < nobj; k++)
-            (*maximum)[k] = NEG_INFINITY;
-    }
-
-    for (k = 0, r = 0; r < rows; r++) {
-        for (int n = 0; n < nobj; n++, k++) {
-            if ((*minimum)[n] > data[k])
-                (*minimum)[n] = data[k];
-            if ((*maximum)[n] < data[k])
-                (*maximum)[n] = data[k];
-        }
-    }
-}
-
 static inline bool
-any_less_than (const double *a, const double *b, int dim)
+any_less_than (const double *a, const double *b, int nobj)
 {
-    for (int d = 0; d < dim; d++)
+    for (int d = 0; d < nobj; d++)
         if (a[d] < b[d])
             return true;
 
@@ -216,43 +151,22 @@ any_less_than (const double *a, const double *b, int dim)
 }
 
 static void
-file_bounds (const char *filename, double **minimum_p, double **maximum_p,
-             int *dim_p)
-{
-    double *data = NULL;
-    int *cumsizes = NULL;
-    int nruns = 0;
-    int dim = *dim_p;
-
-    int err = read_double_data (filename, &data, &dim, &cumsizes, &nruns);
-    if (!filename) filename = stdin_name;
-    handle_read_data_error (err, filename);
-
-    data_bounds (minimum_p, maximum_p, data, dim, cumsizes[nruns - 1]);
-
-    *dim_p = dim;
-
-    free (data);
-    free (cumsizes);
-}
-
-static void
-logarithm_scale (double *points, int dim, int size,
+logarithm_scale (double *points, int nobj, int size,
                  const bool *logarithm)
 {
     int k, d;
 
     assert (logarithm);
 
-    for (d = 0; d < dim; d++) {
+    for (d = 0; d < nobj; d++) {
         if (!logarithm[d]) continue;
         for (k = 0; k < size; k++)
-            points[k * dim + d] = log10(points[k * dim + d]);
+            points[k * nobj + d] = log10(points[k * nobj + d]);
     }
 }
 
 static bool
-force_bounds (double *points, int dim, int *cumsizes, int nsets,
+force_bounds (double *points, int nobj, int *cumsizes, int nsets,
               const double *lbound, const double *ubound)
 {
     int n, n2, k;
@@ -260,8 +174,8 @@ force_bounds (double *points, int dim, int *cumsizes, int nsets,
     bool *outbounds = malloc (sizeof(bool) * size);
     int outbounds_found = -1;
     for (n = size - 1; n >= 0; n--) {
-        if (any_less_than (&points[n * dim], lbound, dim)
-            || any_less_than (ubound, &points[n * dim], dim)) {
+        if (any_less_than (&points[n * nobj], lbound, nobj)
+            || any_less_than (ubound, &points[n * nobj], nobj)) {
             outbounds[n] = true;
             outbounds_found = n;
         } else
@@ -292,8 +206,8 @@ force_bounds (double *points, int dim, int *cumsizes, int nsets,
             if (outbounds[n2]) {
                 n2++; ssizes[k]--;
             } else {
-                memcpy (&points[n * dim], &points[n2 * dim],
-                        sizeof(double) * dim);
+                memcpy (&points[n * nobj], &points[n2 * nobj],
+                        sizeof(double) * nobj);
                 n++, n2++;
             }
         }
@@ -313,7 +227,7 @@ force_bounds (double *points, int dim, int *cumsizes, int nsets,
 
 static bool
 check_nondominated (const char * filename, const double *points,
-                    int dim, const int *cumsizes, int nruns,
+                    int nobj, const int *cumsizes, int nruns,
                     const signed char *minmax, const signed char agree,
                     bool **nondom_p)
 {
@@ -331,7 +245,7 @@ check_nondominated (const char * filename, const double *points,
     for (n = 0, cumsize = 0; n < nruns; cumsize = cumsizes[n], n++) {
         int old_size = cumsizes[n] - cumsize;
         int new_size
-            = find_nondominated_set_agree (&points[dim * cumsize], dim,
+            = find_nondominated_set_agree (&points[nobj * cumsize], nobj,
                                            old_size, minmax, agree,
                                            &nondom[cumsize]);
 
@@ -371,12 +285,12 @@ check_nondominated (const char * filename, const double *points,
 
 static void
 print_file_info (FILE *stream, const char *filename,
-                 int dim, const signed char *minmax)
+                 int nobj, const signed char *minmax)
 {
     /* Print some info about input files.  */
     fprintf (stream, "# file: %s\n", filename);
-    fprintf (stream, "# objectives (%d): ", dim);
-    for (int n = 0; n < dim; n++) {
+    fprintf (stream, "# objectives (%d): ", nobj);
+    for (int n = 0; n < nobj; n++) {
         fprintf (stream, "%c", (minmax[n] < 0) ? '-'
                  : ((minmax[n] > 0) ? '+' : 'i'));
     }
@@ -385,19 +299,19 @@ print_file_info (FILE *stream, const char *filename,
 
 static void
 print_output_header (FILE *stream, const char *filename,
-                     int dim, const signed char *minmax,
+                     int nobj, const signed char *minmax,
                      signed char agree,
                      double lrange, double urange,
                      const double *lbound, const double *ubound,
                      const bool *logarithm)
 {
-    print_file_info (stream, filename, dim, minmax);
+    print_file_info (stream, filename, nobj, minmax);
     fprintf (stream, "# agree: %s\n", (agree < 0) ? "min"
              : ((agree > 0) ? "max" : "no"));
 
     if (logarithm) {
         fprintf (stream, "# logarithm: ");
-        for (int n = 0; n < dim; n++) {
+        for (int n = 0; n < nobj; n++) {
             fprintf (stream, "%c", logarithm[n] ? '1' : '0');
         }
         fprintf (stream, "\n");
@@ -409,56 +323,53 @@ print_output_header (FILE *stream, const char *filename,
                  "\n", lrange, urange);
 
     fprintf (stream, "# lower bound: ");
-    vector_fprintf (stream, lbound, dim);
+    vector_fprintf (stream, lbound, nobj);
     fprintf (stream, "\n");
     fprintf (stream, "# upper bound: ");
-    vector_fprintf (stream, ubound, dim);
+    vector_fprintf (stream, ubound, nobj);
     fprintf (stream, "\n");
 }
 
 static void
 print_input_info (FILE *stream, const char *filename,
-                  int dim, const int *cumsizes, int nruns,
+                  int nobj, const int *cumsizes, int nruns,
                   const signed char *minmax,
                   const double *minimum, const double *maximum)
 {
-    print_file_info (stream, filename, dim, minmax);
+    print_file_info (stream, filename, nobj, minmax);
     fprintf (stream, "# sets: %d\n", nruns);
     fprintf (stream, "# sizes: %d", cumsizes[0]);
     for (int n = 1; n < nruns; n++)
         fprintf (stream, ", %d", cumsizes[n] - cumsizes[n - 1]);
-    fprintf (stream, "\n");
-    fprintf (stream, "# points: %d\n", cumsizes[nruns - 1]);
-    fprintf (stream, "# minimum:");
-    vector_fprintf (stream, minimum, dim);
-    fprintf (stream, "\n");
-    fprintf (stream, "# maximum:");
-    vector_fprintf (stream, maximum, dim);
+    fprintf (stream, "\n# points: %d\n", cumsizes[nruns - 1]);
+    fprintf (stream, "# minimum: ");
+    vector_fprintf (stream, minimum, nobj);
+    fprintf (stream, "\n# maximum: ");
+    vector_fprintf (stream, maximum, nobj);
     fprintf (stream, "\n");
 }
 
 static bool
 process_file (const char *filename,
-              const signed char *minmax, int *dim_p, signed char agree,
+              const signed char *minmax, int *nobj_p, signed char agree,
               double lrange, double urange,
               double *lbound, double *ubound,
               double **minimum_p, double **maximum_p,
-              bool check_minimum, bool check_maximum,
+              bool check_minimum, bool check_maximum, bool maximise_all_flag,
               const bool *logarithm)
 {
     bool *nondom = NULL;
-    bool dominated_found = false;
     bool logarithm_flag = false;
 
     double *points = NULL;
-    int d;
-    int dim = *dim_p;
+    int nobj = *nobj_p;
     int *cumsizes = NULL;
     int nsets = 0;
 
-    int err = read_double_data (filename, &points, &dim, &cumsizes, &nsets);
-    if (!filename) filename = stdin_name;
-    handle_read_data_error (err, filename);
+    handle_read_data_error(
+        read_double_data (filename, &points, &nobj, &cumsizes, &nsets), filename);
+    if (!filename)
+        filename = stdin_name;
 
     if (union_flag) {
         cumsizes[0] = cumsizes[nsets - 1];
@@ -468,24 +379,24 @@ process_file (const char *filename,
     /* Default minmax if not set yet.  */
     bool free_minmax = false;
     if (minmax == NULL) {
-        minmax = read_minmax (NULL, &dim);
+        minmax = maximise_all_flag ? minmax_maximise(nobj) : minmax_minimise(nobj);
         free_minmax = true;
     }
 
     double *minimum = NULL;
     double *maximum = NULL;
-    data_bounds (&minimum, &maximum, points, dim, cumsizes[nsets - 1]);
+    data_bounds (&minimum, &maximum, points, nobj, cumsizes[nsets - 1]);
 
     if (verbose_flag >= 2)
-        print_input_info (stderr, filename, dim, cumsizes, nsets, minmax,
+        print_input_info (stderr, filename, nobj, cumsizes, nsets, minmax,
                           minimum, maximum);
 
     if (lbound == NULL)
         lbound = minimum;
     else if (check_minimum && !force_bounds_flag
-             && any_less_than (minimum, lbound, dim)) {
+             && any_less_than (minimum, lbound, nobj)) {
         errprintf ("%s: found vector smaller than lower bound:", filename);
-        vector_fprintf (stderr, minimum, dim);
+        vector_fprintf (stderr, minimum, nobj);
         fprintf (stderr, "\n");
         exit (EXIT_FAILURE);
     }
@@ -493,28 +404,28 @@ process_file (const char *filename,
     if (ubound == NULL)
         ubound = maximum;
     else if (check_maximum && !force_bounds_flag
-             && any_less_than (ubound, maximum, dim)) {
+             && any_less_than (ubound, maximum, nobj)) {
         errprintf ("%s: found vector larger than upper bound:", filename);
-        vector_fprintf (stderr, maximum, dim);
+        vector_fprintf (stderr, maximum, nobj);
         fprintf (stderr, "\n");
         exit (EXIT_FAILURE);
     }
 
     if (force_bounds_flag) {
-        force_bounds (points, dim, cumsizes, nsets, lbound, ubound);
+        force_bounds (points, nobj, cumsizes, nsets, lbound, ubound);
     }
 
     double *log_lbound = NULL;
     double *log_ubound = NULL;
 
     if (logarithm) {
-        log_lbound = malloc(sizeof(double) * dim);
-        log_ubound = malloc(sizeof(double) * dim);
+        log_lbound = malloc(sizeof(double) * nobj);
+        log_ubound = malloc(sizeof(double) * nobj);
 
-        memcpy (log_lbound, lbound, sizeof(double) * dim);
-        memcpy (log_ubound, ubound, sizeof(double) * dim);
+        memcpy (log_lbound, lbound, sizeof(double) * nobj);
+        memcpy (log_ubound, ubound, sizeof(double) * nobj);
 
-        for (d = 0; d < dim; d++) {
+        for (int d = 0; d < nobj; d++) {
             if (!logarithm[d]) continue;
             log_lbound[d] = log10(lbound[d]);
             log_ubound[d] = log10(ubound[d]);
@@ -524,22 +435,23 @@ process_file (const char *filename,
         if (logarithm_flag) {
             lbound = log_lbound;
             ubound = log_ubound;
-            logarithm_scale (points, dim, cumsizes[nsets - 1], logarithm);
+            logarithm_scale (points, nobj, cumsizes[nsets - 1], logarithm);
         }
     }
 
     if (agree)
-        agree_objectives (points, dim, cumsizes[nsets - 1], minmax, agree);
+        agree_objectives (points, nobj, cumsizes[nsets - 1], minmax, agree);
 
     if (normalise_flag)
-        normalise (points, dim, cumsizes[nsets - 1], minmax, agree,
+        normalise (points, nobj, cumsizes[nsets - 1], minmax, agree,
                    lrange, urange, lbound, ubound);
 
+    bool dominated_found = false;
     /* Check sets.  */
     if (check_flag || filter_flag)
-        if (check_nondominated (filename, points, dim, cumsizes, nsets,
-                                minmax, agree, filter_flag ? &nondom : NULL))
-            dominated_found = true;
+        dominated_found =
+            check_nondominated (filename, points, nobj, cumsizes, nsets,
+                                minmax, agree, filter_flag ? &nondom : NULL);
 
     if (verbose_flag >= 2)
         fprintf (stderr, "# nondominated: %s\n",
@@ -554,19 +466,18 @@ process_file (const char *filename,
         if (filename != stdin_name) {
             outfilename = m_strcat(filename, suffix);
             outfile = fopen (outfilename, "w");
-            if (outfile == NULL) {
-                errprintf ("%s: %s\n", outfilename, strerror(errno));
-                exit(EXIT_FAILURE);
-            }
+            if (outfile == NULL)
+                fatal_error("%s: %s\n", outfilename, strerror(errno));
         }
-        print_output_header (outfile, filename, dim, minmax, agree,
-                             lrange, urange, lbound, ubound,
-                             logarithm);
+        if (verbose_flag)
+            print_output_header (outfile, filename, nobj, minmax, agree,
+                                 lrange, urange, lbound, ubound,
+                                 logarithm);
 
         if (filter_flag && dominated_found)
-            write_sets_filtered (outfile, points, dim, cumsizes, nsets, nondom);
+            write_sets_filtered (outfile, points, nobj, cumsizes, nsets, nondom);
         else
-            write_sets (outfile, points, dim, cumsizes, nsets);
+            write_sets (outfile, points, nobj, cumsizes, nsets);
 
         if (verbose_flag)
             fprintf (stderr, "# %s -> %s\n", filename, outfilename);
@@ -585,7 +496,7 @@ process_file (const char *filename,
 
     *minimum_p = minimum;
     *maximum_p = maximum;
-    *dim_p = dim;
+    *nobj_p = nobj;
 
     if (verbose_flag >= 2)
         fprintf (stderr, "#\n");
@@ -602,41 +513,39 @@ int main(int argc, char *argv[])
     double *upper_bound = NULL;
 
     const signed char *minmax = NULL;
+    bool maximise_all_flag = false;
     const bool *logarithm = NULL;
-    int dim = 0;
+    int nobj = 0;
 
     /* see the man page for getopt_long for an explanation of these fields */
-    static struct option long_options[] = {
+    static const char short_options[] = "hVvqfo:a:n:u:l:Us:b";
+    static const struct option long_options[] = {
         {"help",       no_argument,       NULL, 'h'},
         {"version",    no_argument,       NULL, 'V'},
         {"verbose",    no_argument,       NULL, 'v'},
         {"quiet",      no_argument,       NULL, 'q'},
-
         {"no-check",   no_argument,       NULL, 'c'},
         {"filter",     no_argument,       NULL, 'f'},
         {"force-bounds",no_argument,      NULL, 'b'},
         {"obj",        required_argument, NULL, 'o'},
         {"agree",      required_argument, NULL, 'a'},
+        {"maximise",   no_argument,       NULL, 'M'},
+        {"maximize",   no_argument,       NULL, 'M'},
         {"normalise",  required_argument, NULL, 'n'},
         {"upper-bound",required_argument, NULL, 'u'},
         {"lower-bound",required_argument, NULL, 'l'},
         {"union",      no_argument,       NULL, 'U'},
         {"suffix",     required_argument, NULL, 's'},
         {"log",        required_argument, NULL, 'L'},
-
         {NULL, 0, NULL, 0} /* marks end of list */
     };
     set_program_invocation_short_name(argv[0]);
 
     int opt; /* it's actually going to hold a char */
     int longopt_index;
-    while (0 < (opt = getopt_long (argc, argv, "hVvqfo:a:n:u:l:Us:b",
+    while (0 < (opt = getopt_long (argc, argv, short_options,
                                    long_options, &longopt_index))) {
         switch (opt) {
-        case 'V': // --version
-            version();
-            exit(EXIT_SUCCESS);
-
         case 'q': // --quiet
             verbose_flag = 0;
             break;
@@ -663,12 +572,7 @@ int main(int argc, char *argv[])
             break;
 
         case 'o': // --obj
-            minmax = read_minmax (optarg, &dim);
-            if (minmax == NULL) {
-                errprintf ("invalid argument '%s' for -o, --obj"
-                           ", it should be a sequence of '+' or '-'\n", optarg);
-                exit(EXIT_FAILURE);
-            }
+            minmax = parse_cmdline_minmax(minmax, optarg, &nobj);
             break;
 
         case 'a': // --agree
@@ -677,33 +581,32 @@ int main(int argc, char *argv[])
             else if (!strcmp (optarg, "min"))
                 agree = -1;
             else {
-                errprintf ("invalid argument '%s' for -a, --agree"
-                           ", it should be either \'min\' or \'max\'\n",
-                           optarg);
-                exit (EXIT_FAILURE);
+                fatal_error("invalid argument '%s' for -a, --agree"
+                            ", it should be either \'min\' or \'max\'\n",
+                            optarg);
             }
             break;
 
+        case 'M': // --maximise
+            maximise_all_flag = true;
+            break;
+
         case 'n': // --normalise
             normalise_flag = true;
             if (!read_range (optarg, &lower_range, &upper_range)) {
-                errprintf ("invalid range '%s' for -n, --normalise"
-                           ", use for example -n \"1 2\"\n", optarg);
-                exit (EXIT_FAILURE);
-
+                fatal_error("invalid range '%s' for -n, --normalise"
+                            ", use for example -n \"1 2\"\n", optarg);
             } else if (lower_range >= upper_range) {
-                errprintf ("lower range must be smaller than upper range"
-                           " for -n, --normalise\n");
-                exit (EXIT_FAILURE);
+                fatal_error ("lower range must be smaller than upper range for -n, --normalise\n");
             }
             break;
 
         case 'u': // --upper-bound
-            upper_bound = robust_read_point (optarg, &dim, "invalid upper bound point '%s'");
+            upper_bound = robust_read_point (optarg, &nobj, "invalid upper bound point '%s'");
             break;
 
         case 'l': // --lower-bound
-            lower_bound = robust_read_point (optarg, &dim, "invalid lower bound point '%s'");
+            lower_bound = robust_read_point (optarg, &nobj, "invalid lower bound point '%s'");
             break;
 
         case 's': // --suffix
@@ -711,71 +614,58 @@ int main(int argc, char *argv[])
             break;
 
         case 'L': // --log
-            logarithm = read_bitvector (optarg, &dim);
-            if (logarithm == NULL) {
-                errprintf ("invalid argument to --log '%s'", optarg);
-                exit (EXIT_FAILURE);
-            }
+            logarithm = read_bitvector (optarg, &nobj);
+            if (logarithm == NULL)
+                fatal_error("invalid argument to --log '%s'", optarg);
             break;
 
-        case '?':
-            // getopt prints an error message right here
-            fprintf(stderr, "Try `%s --help' for more information.\n",
-                    program_invocation_short_name);
-            exit(EXIT_FAILURE);
-        case 'h':
-            usage();
-            exit(EXIT_SUCCESS);
-
-        default: // should never happen
-            abort();
+        default:
+            default_cmdline_handler(opt);
         }
     }
 
     if (lower_bound && upper_bound
-        && any_less_than (upper_bound, lower_bound, dim)) {
-        errprintf ("upper bound must be higher than lower bound.");
-        exit (EXIT_FAILURE);
+        && any_less_than (upper_bound, lower_bound, nobj)) {
+        fatal_error("upper bound must be higher than lower bound.");
     }
 
     int numfiles = argc - optind;
     double *minimum = NULL;
     double *maximum = NULL;
-    bool dominated_found = false;
 
     if (numfiles <= 1) {/* <= 0 means: No input files: read stdin.  */
-        dominated_found =
+        bool dominated_found =
             process_file ((numfiles == 1) ? argv[optind] : NULL,
-                          minmax, &dim, agree,
+                          minmax, &nobj, agree,
                           lower_range, upper_range,
                           lower_bound, upper_bound,
                           &minimum, &maximum,
                           /*check_minimum=*/true, /*check_maximum=*/true,
-                          logarithm);
+                          maximise_all_flag, logarithm);
         free(minimum);
         free(maximum);
         free((void*)minmax);
-        return dominated_found;
+        return filter_flag ? EXIT_SUCCESS : dominated_found;
     }
 
     int k;
-
+    bool dominated_found = false;
     if (!lower_bound || !upper_bound) {
         /* Calculate the bounds among all input files.  */
         minimum = NULL, maximum = NULL;
         for (k = 0; k < numfiles; k++)
-            file_bounds (argv[optind + k], &minimum, &maximum, &dim);
+            file_bounds (argv[optind + k], &minimum, &maximum, &nobj);
 
         k = 0;
     } else {
         /* If the bounds were given, initialize minimum and maximum.  */
-        if (process_file (argv[optind], minmax, &dim, agree,
+        dominated_found =
+            process_file (argv[optind], minmax, &nobj, agree,
                           lower_range, upper_range,
                           lower_bound, upper_bound,
                           &minimum, &maximum,
                           /*check_minimum=*/true, /*check_maximum=*/true,
-                          logarithm))
-            dominated_found = true;
+                          maximise_all_flag, logarithm);
         k = 1;
     }
 
@@ -783,19 +673,19 @@ int main(int argc, char *argv[])
         double *tmp_maximum = NULL;
         double *tmp_minimum = NULL;
 
-        if (process_file (argv[optind + k], minmax, &dim, agree,
+        if (process_file (argv[optind + k], minmax, &nobj, agree,
                           lower_range, upper_range,
                               (lower_bound) ? lower_bound : minimum,
                           (upper_bound) ? upper_bound : maximum,
                           &tmp_minimum, &tmp_maximum,
                           lower_bound != NULL, upper_bound != NULL,
-                          logarithm))
+                          maximise_all_flag, logarithm))
             dominated_found = true;
 
         /* If the bounds were given, the real minimum and maximum
            must be calculated as we process the files.  */
         if (lower_bound && upper_bound)
-            for (int n = 0; n < dim; n++) {
+            for (int n = 0; n < nobj; n++) {
                 if (minimum[n] > tmp_minimum[n])
                     minimum[n] = tmp_minimum[n];
                 if (maximum[n] < tmp_maximum[n])
@@ -807,14 +697,14 @@ int main(int argc, char *argv[])
     if (verbose_flag) {
         printf ("# Total files: %d\n", numfiles);
         printf ("# Total minimum:");
-        vector_printf (minimum, dim);
+        vector_printf (minimum, nobj);
         printf ("\n");
         printf ("# Total maximum:");
-        vector_printf (maximum, dim);
+        vector_printf (maximum, nobj);
         printf ("\n");
         printf ("# Nondominated: %s\n", dominated_found ? "FALSE": "TRUE");
     }
     free(minimum);
     free(maximum);
-    return dominated_found;
+    return filter_flag ? EXIT_SUCCESS : dominated_found;
 }
